local gr = love.graphics
local pi,sin,cos,pow,abs = math.pi, math.sin, math.cos, math.pow, math.abs

function love.load()
	love.window.setTitle( 'SuperEllipse' )
	par = {
		n		= {.5, .5, .5},	-- powers
		m		= 24,			-- number of points
		a		= { 1, 1},		-- x & y relative size
		steps	= 80,
		radius	= 150,
		color	= { 155, 10, 90 }
		}
	Super,SuperUpdate = newSuper( 400, 300, par )
	end

function love.update( dt )
	SuperUpdate( dt )
	end

function love.draw()
	Super()
	showSettings( par, 10, 30 )
	end

function love.keypressed( k, r )
	if k == "escape" then love.event.quit() end
	if k == 'm' then par.m = par.m+1 Super,SuperUpdate = newSuper(par.cx, par.cy, par ) end
	if k == 'n' then par.m = par.m-1 Super,SuperUpdate = newSuper(par.cx, par.cy, par ) end
	if k == 'k' then par.n={par.n[1]+.1,par.n[2]+.1,par.n[3]+.1} Super,SuperUpdate = newSuper(par.cx, par.cy, par ) end
	if k == 'j' then par.n={par.n[1]-.1,par.n[2]-.1,par.n[3]-.1} Super,SuperUpdate = newSuper(par.cx, par.cy, par ) end
	if k == 's' then par.steps = par.steps+2 Super,SuperUpdate = newSuper(par.cx, par.cy, par ) end
	if k == 'a' then par.steps = par.steps-2 Super,SuperUpdate = newSuper(par.cx, par.cy, par ) end
	end

function showSettings( tab, x, y )
	gr.setColor( 255, 255, 255 )
	for i, j in pairs( tab ) do
		gr.print(i..' = '..concat(j),x,y)
		y = y + 30
		end
	gr.print( 'Keys that change: "m" (nm), "n" (jk)', 10, 550 )
	end

function concat( tab )
	if type( tab ) == 'table' then
		str = tab[1]
		for i = 2, #tab do  str = str..', '..tab[i] end
		else
		str =  tab
		end
	return str
	end

function newSuper( cx, cy, par)
	local defaults	= { n={.5,.5,.5}, m=24, a={1,1}, steps=80, radius=100, rot = 0, color = {0,255,0} }
	local par	= setmetatable( par or defaults, { __index = defaults } )
	par.cx, par.cy = cx, cy
	local function spshpe( theta )
		local formula = pow((pow(abs((1 / par.a[1]) * cos(theta * par.m / 4)), par.n[2])) + (pow(math.abs((1 / par.a[2]) * sin(theta *par.m / 4)), par.n[3])), 1 / par.n[1])
		return formula == 0 and 0 or 1 / formula
		end
	local pt = {}
	for i = 0, 2*math.pi, 2*math.pi / par.steps do
		pt[#pt+1] = { spshpe(i) * cos(i),  spshpe(i) * sin(i) }
		end
	return
		function(  sz )
			local sz        = sz or par.radius
			local rot       = par.rot or 0
			local pt2       = polyRot( pt, cx, cy, rot, sz )
			local triangles = love.math.triangulate( pt2 )
			gr.setColor( par.color )
			for i = 1, #triangles do gr.polygon( 'fill', unpack(triangles[i] ) ) end
			end,
		function( tick ) par.rot = par.rot + tick end
	end

function polyRot( v, x, y, rot, scale )
	local scale = scale or 1
	local c, s = cos( rot ), sin( rot )
	local tab = {}
	for i = 1, #v do
		tab[#tab+1] = scale * ( c * v[i][1] - s * v[i][2] ) + x
		tab[#tab+1] = scale * ( s * v[i][1] + c * v[i][2] ) + y
		end
	return tab
	end
